[![FactSet](https://raw.githubusercontent.com/factset/enterprise-sdk/main/docs/images/factset-logo.svg)](https://www.factset.com)

# Symbology client library for Python

[![API Version](https://img.shields.io/badge/api-v2.1.3-blue)]()
[![PyPi](https://img.shields.io/pypi/v/fds.sdk.Symbology)](https://pypi.org/project/fds.sdk.Symbology/)
[![Apache-2 license](https://img.shields.io/badge/license-Apache2-brightgreen.svg)](https://www.apache.org/licenses/LICENSE-2.0)

The FactSet Symbology API provides symbol resolution services, allowing clients to translate market identifiers into various symbology types such as, FactSet Permanent Identifiers, CUSIP, ISIN, SEDOL, Tickers, and Bloomberg FIGIs. <p>Factset's Symbology API sits at the center of its hub-and-spoke data model, enabling you to quickly harmonize the expanding catalog of Content APIs. Translate market IDs into CUSIP, SEDOL, ISIN, Tickers as of a point in time or for the entire history of the requested id allowing Data Management workflows to normalize ids over time.</p>


This Python package is automatically generated by the [OpenAPI Generator](https://openapi-generator.tech) project:

- API version: 2.1.3
- SDK version: 1.0.16
- Build package: org.openapitools.codegen.languages.PythonClientCodegen

## Requirements

* Python >= 3.7

## Installation

### Poetry

```shell
poetry add fds.sdk.utils fds.sdk.Symbology==1.0.16
```

### pip

```shell
pip install fds.sdk.utils fds.sdk.Symbology==1.0.16
```

## Usage

1. [Generate authentication credentials](../../../../README.md#authentication).
2. Setup Python environment.
   1. Install and activate python 3.7+. If you're using [pyenv](https://github.com/pyenv/pyenv):

      ```sh
      pyenv install 3.9.7
      pyenv shell 3.9.7
      ```

   2. (optional) [Install poetry](https://python-poetry.org/docs/#installation).
3. [Install dependencies](#installation).
4. Run the following:

> [!IMPORTANT]
> The parameter variables defined below are just examples and may potentially contain non valid values. Please replace them with valid values.

### Example Code

```python
from fds.sdk.utils.authentication import ConfidentialClient

import fds.sdk.Symbology
from fds.sdk.Symbology.api import bloomberg_figi_api
from fds.sdk.Symbology.models import *
from dateutil.parser import parse as dateutil_parser
from pprint import pprint

# See configuration.py for a list of all supported configuration parameters.

# Examples for each supported authentication method are below,
# choose one that satisfies your use case.

# (Preferred) OAuth 2.0: FactSetOAuth2
# See https://github.com/FactSet/enterprise-sdk#oauth-20
# for information on how to create the app-config.json file
#
# The confidential client instance should be reused in production environments.
# See https://github.com/FactSet/enterprise-sdk-utils-python#authentication
# for more information on using the ConfidentialClient class
configuration = fds.sdk.Symbology.Configuration(
    fds_oauth_client=ConfidentialClient('/path/to/app-config.json')
)

# Basic authentication: FactSetApiKey
# See https://github.com/FactSet/enterprise-sdk#api-key
# for information how to create an API key
# configuration = fds.sdk.Symbology.Configuration(
#     username='USERNAME-SERIAL',
#     password='API-KEY'
# )

# Enter a context with an instance of the API client
with fds.sdk.Symbology.ApiClient(configuration) as api_client:
    # Create an instance of the API class
    api_instance = bloomberg_figi_api.BloombergFIGIApi(api_client)
    bloomberg_translation_request = BloombergTranslationRequest(
        ids=GetIds(["GOOGL-US"]),
    ) # BloombergTranslationRequest | Request Body for Bloomberg FIGIs.

    try:
        # Translate market security symbols into Bloomberg Identifiers.
        # example passing only required values which don't have defaults set
        api_response = api_instance.batch_translate_bloomberg(bloomberg_translation_request)

        pprint(api_response)
    except fds.sdk.Symbology.ApiException as e:
        print("Exception when calling BloombergFIGIApi->batch_translate_bloomberg: %s\n" % e)

    # # Get response, http status code and response headers
    # try:
    #     # Translate market security symbols into Bloomberg Identifiers.
    #     api_response, http_status_code, response_headers = api_instance.batch_translate_bloomberg_with_http_info(bloomberg_translation_request)


    #     pprint(api_response)
    #     pprint(http_status_code)
    #     pprint(response_headers)
    # except fds.sdk.Symbology.ApiException as e:
    #     print("Exception when calling BloombergFIGIApi->batch_translate_bloomberg: %s\n" % e)

    # # Get response asynchronous
    # try:
    #     # Translate market security symbols into Bloomberg Identifiers.
    #     async_result = api_instance.batch_translate_bloomberg_async(bloomberg_translation_request)
    #     api_response = async_result.get()


    #     pprint(api_response)
    # except fds.sdk.Symbology.ApiException as e:
    #     print("Exception when calling BloombergFIGIApi->batch_translate_bloomberg: %s\n" % e)

    # # Get response, http status code and response headers asynchronous
    # try:
    #     # Translate market security symbols into Bloomberg Identifiers.
    #     async_result = api_instance.batch_translate_bloomberg_with_http_info_async(bloomberg_translation_request)
    #     api_response, http_status_code, response_headers = async_result.get()


    #     pprint(api_response)
    #     pprint(http_status_code)
    #     pprint(response_headers)
    # except fds.sdk.Symbology.ApiException as e:
    #     print("Exception when calling BloombergFIGIApi->batch_translate_bloomberg: %s\n" % e)

```

### Using Pandas

To convert an API response to a Pandas DataFrame, it is necessary to transform it first to a dictionary.
```python
import pandas as pd

response_dict = api_response.to_dict()['data']

simple_json_response = pd.DataFrame(response_dict)
nested_json_response = pd.json_normalize(response_dict)
```

### Debugging

The SDK uses the standard library [`logging`](https://docs.python.org/3/library/logging.html#module-logging) module.

Setting `debug` to `True` on an instance of the `Configuration` class sets the log-level of related packages to `DEBUG`
and enables additional logging in Pythons [HTTP Client](https://docs.python.org/3/library/http.client.html).

**Note**: This prints out sensitive information (e.g. the full request and response). Use with care.

```python
import logging
import fds.sdk.Symbology

logging.basicConfig(level=logging.DEBUG)

configuration = fds.sdk.Symbology.Configuration(...)
configuration.debug = True
```

### Configure a Proxy

You can pass proxy settings to the Configuration class:

* `proxy`: The URL of the proxy to use.
* `proxy_headers`: a dictionary to pass additional headers to the proxy (e.g. `Proxy-Authorization`).

```python
import fds.sdk.Symbology

configuration = fds.sdk.Symbology.Configuration(
    # ...
    proxy="http://secret:password@localhost:5050",
    proxy_headers={
        "Custom-Proxy-Header": "Custom-Proxy-Header-Value"
    }
)
```

### Custom SSL Certificate

TLS/SSL certificate verification can be configured with the following Configuration parameters:

* `ssl_ca_cert`: a path to the certificate to use for verification in `PEM` format.
* `verify_ssl`: setting this to `False` disables the verification of certificates.
  Disabling the verification is not recommended, but it might be useful during
  local development or testing.

```python
import fds.sdk.Symbology

configuration = fds.sdk.Symbology.Configuration(
    # ...
    ssl_ca_cert='/path/to/ca.pem'
)
```

### Request Retries

In case the request retry behaviour should be customized, it is possible to pass a `urllib3.Retry` object to the `retry` property of the Configuration.

```python
from urllib3 import Retry
import fds.sdk.Symbology

configuration = fds.sdk.Symbology.Configuration(
    # ...
)

configuration.retries = Retry(total=3, status_forcelist=[500, 502, 503, 504])
```


## Documentation for API Endpoints

All URIs are relative to *https://api.factset.com/content*

Class | Method | HTTP request | Description
------------ | ------------- | ------------- | -------------
*BloombergFIGIApi* | [**batch_translate_bloomberg**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Symbology/v2/docs/BloombergFIGIApi.md#batch_translate_bloomberg) | **POST** /symbology/v2/bloomberg | Translate market security symbols into Bloomberg Identifiers.
*BloombergFIGIApi* | [**translate_bloomberg**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Symbology/v2/docs/BloombergFIGIApi.md#translate_bloomberg) | **GET** /symbology/v2/bloomberg | Translate market security symbols into Bloomberg Identifiers.
*CUSIPApi* | [**batch_cusip_history**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Symbology/v2/docs/CUSIPApi.md#batch_cusip_history) | **POST** /symbology/v2/cusip-history | Retrieve the full history or as of a specific date of CUSIP changes for the requested ID(s).
*CUSIPApi* | [**batch_translate_cusips**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Symbology/v2/docs/CUSIPApi.md#batch_translate_cusips) | **POST** /symbology/v2/cusip | Translate market security symbols into CUSIP - Current Only
*CUSIPApi* | [**cusip_history**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Symbology/v2/docs/CUSIPApi.md#cusip_history) | **GET** /symbology/v2/cusip-history | Retrieve the full history or as of a requested date of CUSIP changes for the requested ID(s).
*CUSIPApi* | [**translate_cusip**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Symbology/v2/docs/CUSIPApi.md#translate_cusip) | **GET** /symbology/v2/cusip | Translate market security symbols into CUSIPs - Current Only
*FactSetApi* | [**batch_translate_fact_set**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Symbology/v2/docs/FactSetApi.md#batch_translate_fact_set) | **POST** /symbology/v2/factset | Translate market security symbols into FactSet Permanent Identifiers
*FactSetApi* | [**batch_translate_fact_set_advanced**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Symbology/v2/docs/FactSetApi.md#batch_translate_fact_set_advanced) | **POST** /symbology/v2/factset-advanced | Translate advanced symbol types into FactSet Permanent Identifiers
*FactSetApi* | [**translate_factset**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Symbology/v2/docs/FactSetApi.md#translate_factset) | **GET** /symbology/v2/factset | Translate market security symbols into FactSet Permanent Identifiers.
*FactSetApi* | [**translate_factset_advanced**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Symbology/v2/docs/FactSetApi.md#translate_factset_advanced) | **GET** /symbology/v2/factset-advanced | Translate advanced symbol types into FactSet Permanent Identifiers
*ISINApi* | [**batch_isin_history**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Symbology/v2/docs/ISINApi.md#batch_isin_history) | **POST** /symbology/v2/isin-history | Translate Market IDs into ISINS as of a specific date or receive the full history of changes.
*ISINApi* | [**batch_translate_isin**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Symbology/v2/docs/ISINApi.md#batch_translate_isin) | **POST** /symbology/v2/isin | Translate market security symbols into ISIN. - Current Only
*ISINApi* | [**isin_history**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Symbology/v2/docs/ISINApi.md#isin_history) | **GET** /symbology/v2/isin-history | Translate Market IDs into ISINS as of a specific date or receive the full history of changes.
*ISINApi* | [**translate_isin**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Symbology/v2/docs/ISINApi.md#translate_isin) | **GET** /symbology/v2/isin | Translate market security symbols into ISIN. - Current Only
*SEDOLApi* | [**batch_sedol_history**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Symbology/v2/docs/SEDOLApi.md#batch_sedol_history) | **POST** /symbology/v2/sedol-history | Retrieve the full history or as of a specific date of SEDOL changes for a requested ID.
*SEDOLApi* | [**batch_translate_sedol**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Symbology/v2/docs/SEDOLApi.md#batch_translate_sedol) | **POST** /symbology/v2/sedol | Translate market security symbols into SEDOL. - Current Only
*SEDOLApi* | [**sedol_history**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Symbology/v2/docs/SEDOLApi.md#sedol_history) | **GET** /symbology/v2/sedol-history | Retrieve the full history or as of a specific date of SEDOL changes for the requested ID(s).
*SEDOLApi* | [**translate_sedol**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Symbology/v2/docs/SEDOLApi.md#translate_sedol) | **GET** /symbology/v2/sedol | Translate market security symbols into SEDOL. - Current Only
*TickerApi* | [**batch_ticker_history**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Symbology/v2/docs/TickerApi.md#batch_ticker_history) | **POST** /symbology/v2/ticker-history | Returns the full history of Ticker-Exchange or Ticker-Region changes for a requested security.
*TickerApi* | [**batch_translate_ticker**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Symbology/v2/docs/TickerApi.md#batch_translate_ticker) | **POST** /symbology/v2/ticker | Returns the Ticker-Exchange or Ticker-Region for a given security.
*TickerApi* | [**ticker_history**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Symbology/v2/docs/TickerApi.md#ticker_history) | **GET** /symbology/v2/ticker-history | Returns the full history of Ticker-Exchange or Ticker-Region changes for a requested security.
*TickerApi* | [**translate_ticker**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Symbology/v2/docs/TickerApi.md#translate_ticker) | **GET** /symbology/v2/ticker | Returns the Ticker-Exchange or Ticker-Region for a given security. - Current Only


## Documentation For Models

 - [BloombergTranslation](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Symbology/v2/docs/BloombergTranslation.md)
 - [BloombergTranslationRequest](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Symbology/v2/docs/BloombergTranslationRequest.md)
 - [BloombergTranslationResponse](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Symbology/v2/docs/BloombergTranslationResponse.md)
 - [CusipHistoryResponse](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Symbology/v2/docs/CusipHistoryResponse.md)
 - [CusipHistoryTranslation](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Symbology/v2/docs/CusipHistoryTranslation.md)
 - [CusipHistoryTranslationRequest](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Symbology/v2/docs/CusipHistoryTranslationRequest.md)
 - [CusipTranslation](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Symbology/v2/docs/CusipTranslation.md)
 - [CusipTranslationRequest](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Symbology/v2/docs/CusipTranslationRequest.md)
 - [CusipTranslationResponse](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Symbology/v2/docs/CusipTranslationResponse.md)
 - [ErrorResponse](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Symbology/v2/docs/ErrorResponse.md)
 - [ErrorResponseSubErrors](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Symbology/v2/docs/ErrorResponseSubErrors.md)
 - [FactsetTranslation](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Symbology/v2/docs/FactsetTranslation.md)
 - [FactsetTranslationAdvanced](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Symbology/v2/docs/FactsetTranslationAdvanced.md)
 - [FactsetTranslationAdvancedRequest](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Symbology/v2/docs/FactsetTranslationAdvancedRequest.md)
 - [FactsetTranslationAdvancedResponse](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Symbology/v2/docs/FactsetTranslationAdvancedResponse.md)
 - [FactsetTranslationRequest](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Symbology/v2/docs/FactsetTranslationRequest.md)
 - [FactsetTranslationResponse](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Symbology/v2/docs/FactsetTranslationResponse.md)
 - [GetAdvancedIds](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Symbology/v2/docs/GetAdvancedIds.md)
 - [GetIds](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Symbology/v2/docs/GetIds.md)
 - [GetSymbolType](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Symbology/v2/docs/GetSymbolType.md)
 - [IsinHistoryTranslation](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Symbology/v2/docs/IsinHistoryTranslation.md)
 - [IsinHistoryTranslationRequest](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Symbology/v2/docs/IsinHistoryTranslationRequest.md)
 - [IsinHistoryTranslationResponse](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Symbology/v2/docs/IsinHistoryTranslationResponse.md)
 - [IsinTranslation](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Symbology/v2/docs/IsinTranslation.md)
 - [IsinTranslationRequest](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Symbology/v2/docs/IsinTranslationRequest.md)
 - [IsinTranslationResponse](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Symbology/v2/docs/IsinTranslationResponse.md)
 - [SedolHistoryTranslation](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Symbology/v2/docs/SedolHistoryTranslation.md)
 - [SedolHistoryTranslationRequest](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Symbology/v2/docs/SedolHistoryTranslationRequest.md)
 - [SedolHistoryTranslationResponse](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Symbology/v2/docs/SedolHistoryTranslationResponse.md)
 - [SedolTranslation](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Symbology/v2/docs/SedolTranslation.md)
 - [SedolTranslationRequest](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Symbology/v2/docs/SedolTranslationRequest.md)
 - [SedolTranslationResponse](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Symbology/v2/docs/SedolTranslationResponse.md)
 - [TickerHistoryTranslation](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Symbology/v2/docs/TickerHistoryTranslation.md)
 - [TickerHistoryTranslationRequest](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Symbology/v2/docs/TickerHistoryTranslationRequest.md)
 - [TickerHistoryTranslationResponse](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Symbology/v2/docs/TickerHistoryTranslationResponse.md)
 - [TickerTranslation](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Symbology/v2/docs/TickerTranslation.md)
 - [TickerTranslationRequest](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Symbology/v2/docs/TickerTranslationRequest.md)
 - [TickerTranslationResponse](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Symbology/v2/docs/TickerTranslationResponse.md)


## Documentation For Authorization


## FactSetApiKey

- **Type**: HTTP basic authentication


## FactSetOAuth2

- **Type**: OAuth
- **Flow**: application
- **Authorization URL**: 
- **Scopes**: N/A


## Notes for Large OpenAPI documents
If the OpenAPI document is large, imports in fds.sdk.Symbology.apis and fds.sdk.Symbology.models may fail with a
RecursionError indicating the maximum recursion limit has been exceeded. In that case, there are a couple of solutions:

Solution 1:
Use specific imports for apis and models like:
- `from fds.sdk.Symbology.api.default_api import DefaultApi`
- `from fds.sdk.Symbology.model.pet import Pet`

Solution 2:
Before importing the package, adjust the maximum recursion limit as shown below:
```
import sys
sys.setrecursionlimit(1500)
import fds.sdk.Symbology
from fds.sdk.Symbology.apis import *
from fds.sdk.Symbology.models import *
```

## Contributing

Please refer to the [contributing guide](../../../../CONTRIBUTING.md).

## Copyright

Copyright 2022 FactSet Research Systems Inc

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

